// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Redis;
using Microsoft.Azure.Management.Redis.Models;

namespace Microsoft.Azure.Management.Redis
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure Redis Cache Management
    /// Service
    /// </summary>
    public static partial class RedisOperationsExtensions
    {
        /// <summary>
        /// Create a redis cache, or replace (overwrite/recreate, with
        /// potential downtime) an existing cache
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate redis operation.
        /// </param>
        /// <returns>
        /// The response of CreateOrUpdate redis operation.
        /// </returns>
        public static RedisCreateOrUpdateResponse CreateOrUpdate(this IRedisOperations operations, string resourceGroupName, string name, RedisCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).CreateOrUpdateAsync(resourceGroupName, name, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create a redis cache, or replace (overwrite/recreate, with
        /// potential downtime) an existing cache
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate redis operation.
        /// </param>
        /// <returns>
        /// The response of CreateOrUpdate redis operation.
        /// </returns>
        public static Task<RedisCreateOrUpdateResponse> CreateOrUpdateAsync(this IRedisOperations operations, string resourceGroupName, string name, RedisCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, name, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a redis cache. This operation takes a while to complete.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).DeleteAsync(resourceGroupName, name);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a redis cache. This operation takes a while to complete.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return operations.DeleteAsync(resourceGroupName, name, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a redis cache (resource description).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// The response of GET redis operation.
        /// </returns>
        public static RedisGetResponse Get(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).GetAsync(resourceGroupName, name);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a redis cache (resource description).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// The response of GET redis operation.
        /// </returns>
        public static Task<RedisGetResponse> GetAsync(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return operations.GetAsync(resourceGroupName, name, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets all redis caches in a resource group (if provided) otherwise
        /// all in subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Optional. The name of the resource group.
        /// </param>
        /// <returns>
        /// The response of list redis operation.
        /// </returns>
        public static RedisListResponse List(this IRedisOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).ListAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets all redis caches in a resource group (if provided) otherwise
        /// all in subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Optional. The name of the resource group.
        /// </param>
        /// <returns>
        /// The response of list redis operation.
        /// </returns>
        public static Task<RedisListResponse> ListAsync(this IRedisOperations operations, string resourceGroupName)
        {
            return operations.ListAsync(resourceGroupName, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve a redis cache's access keys. This operation requires write
        /// permission to the cache resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// The response of redis list keys operation.
        /// </returns>
        public static RedisListKeysResponse ListKeys(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).ListKeysAsync(resourceGroupName, name);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve a redis cache's access keys. This operation requires write
        /// permission to the cache resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <returns>
        /// The response of redis list keys operation.
        /// </returns>
        public static Task<RedisListKeysResponse> ListKeysAsync(this IRedisOperations operations, string resourceGroupName, string name)
        {
            return operations.ListKeysAsync(resourceGroupName, name, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets all redis caches using next link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// The response of list redis operation.
        /// </returns>
        public static RedisListResponse ListNext(this IRedisOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets all redis caches using next link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// The response of list redis operation.
        /// </returns>
        public static Task<RedisListResponse> ListNextAsync(this IRedisOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Regenerate redis cache's access keys. This operation requires write
        /// permission to the cache resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <param name='parameters'>
        /// Required. Specifies which key to reset.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse RegenerateKey(this IRedisOperations operations, string resourceGroupName, string name, RedisRegenerateKeyParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRedisOperations)s).RegenerateKeyAsync(resourceGroupName, name, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Regenerate redis cache's access keys. This operation requires write
        /// permission to the cache resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Redis.IRedisOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the redis cache.
        /// </param>
        /// <param name='parameters'>
        /// Required. Specifies which key to reset.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> RegenerateKeyAsync(this IRedisOperations operations, string resourceGroupName, string name, RedisRegenerateKeyParameters parameters)
        {
            return operations.RegenerateKeyAsync(resourceGroupName, name, parameters, CancellationToken.None);
        }
    }
}
