// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Sql;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// The Windows Azure SQL Database management API provides a RESTful set of
    /// web services that interact with Windows Azure SQL Database services to
    /// manage your databases. The API enables users to create, retrieve,
    /// update, and delete databases and servers.
    /// </summary>
    public static partial class ServerOperationsExtensions
    {
        /// <summary>
        /// Creates a new Azure SQL Database server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating a
        /// database.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static ServerGetResponse CreateOrUpdate(this IServerOperations operations, string resourceGroupName, string serverName, ServerCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).CreateOrUpdateAsync(resourceGroupName, serverName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a new Azure SQL Database server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating a
        /// database.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static Task<ServerGetResponse> CreateOrUpdateAsync(this IServerOperations operations, string resourceGroupName, string serverName, ServerCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, serverName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).DeleteAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return operations.DeleteAsync(resourceGroupName, serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server to retrieve.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static ServerGetResponse Get(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).GetAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the server to retrieve.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static Task<ServerGetResponse> GetAsync(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return operations.GetAsync(resourceGroupName, serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static ServerListResponse List(this IServerOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).ListAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database Server request.
        /// </returns>
        public static Task<ServerListResponse> ListAsync(this IServerOperations operations, string resourceGroupName)
        {
            return operations.ListAsync(resourceGroupName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Server usage.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which the
        /// Azure SQL Databases are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database Server metrics
        /// request.
        /// </returns>
        public static ServerMetricListResponse ListUsages(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerOperations)s).ListUsagesAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Server usage.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.Azure.Management.Sql.IServerOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which the
        /// Azure SQL Databases are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database Server metrics
        /// request.
        /// </returns>
        public static Task<ServerMetricListResponse> ListUsagesAsync(this IServerOperations operations, string resourceGroupName, string serverName)
        {
            return operations.ListUsagesAsync(resourceGroupName, serverName, CancellationToken.None);
        }
    }
}
