// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.StreamAnalytics.Models;

namespace Microsoft.Azure.Management.StreamAnalytics
{
    /// <summary>
    /// Operations for managing the transformation definition of the stream
    /// analytics job.
    /// </summary>
    public partial interface ITransformationOperations
    {
        /// <summary>
        /// Create or update a transformation for a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// The parameters required to create or update a transformation for
        /// the stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the transformation create operation.
        /// </returns>
        Task<TransformationCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, string jobName, TransformationCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Create or update a transformation for a stream analytics job. The
        /// raw json content will be used.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// The name of the stream analytics job.
        /// </param>
        /// <param name='transformationName'>
        /// The name of the transformation for the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// The parameters required to create or update a transformation for
        /// the stream analytics job. It is in json format.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the transformation create operation.
        /// </returns>
        Task<TransformationCreateOrUpdateResponse> CreateOrUpdateWithRawJsonContentAsync(string resourceGroupName, string jobName, string transformationName, TransformationCreateOrUpdateWithRawJsonContentParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Get the transformation for a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// The name of the stream analytics job.
        /// </param>
        /// <param name='transformationName'>
        /// The name of the transformation for the stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the transformation get operation.
        /// </returns>
        Task<TransformationsGetResponse> GetAsync(string resourceGroupName, string jobName, string transformationName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Update an transformation for a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// The name of the stream analytics job.
        /// </param>
        /// <param name='transformationName'>
        /// The name of the transformation for the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// The parameters required to update an transformation for the stream
        /// analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the transformation patch operation.
        /// </returns>
        Task<TransformationPatchResponse> PatchAsync(string resourceGroupName, string jobName, string transformationName, TransformationPatchParameters parameters, CancellationToken cancellationToken);
    }
}
