// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute
{
    /// <summary>
    /// The Service Management API includes operations for managing the service
    /// and virtual machine extension images in your publisher subscription.
    /// </summary>
    public partial interface IExtensionImageOperations
    {
        /// <summary>
        /// Register a new extension. An extension is identified by the
        /// combination of its ProviderNamespace and Type (case-sensitive
        /// string). It is not allowed to register an extension with the same
        /// identity (i.e. combination of ProviderNamespace and Type) of an
        /// already-registered extension. To register new version of an
        /// existing extension, the Update Extension API should be used.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Register Virtual Machine Extension Image
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> BeginRegisteringAsync(ExtensionImageRegisterParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Unregister a version of an extension that was previously registered
        /// using either the Register Extension or Update Extension APIs. An
        /// extension version is identified by the combination of its
        /// ProviderNamespace, Type and Version which are specified when
        /// registering the extension. Unregistering is only allowed for
        /// internal extensions, that is, the extensions for which the
        /// IsInternalExtension field is set to 'true' during registration or
        /// during an update. There is a quota (15) on the number of
        /// extensions that can be registered per subscription. If your
        /// subscription runs out of quota, you will wither need to unregister
        /// some of the internal extensions or contact Azure (same email used
        /// to become a publisher)  to increase the quota.
        /// </summary>
        /// <param name='providerNamespace'>
        /// The provider namespace of the extension image to unregister.
        /// </param>
        /// <param name='type'>
        /// The type of the extension image to unregister.
        /// </param>
        /// <param name='version'>
        /// The version of the extension image to unregister.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> BeginUnregisteringAsync(string providerNamespace, string type, string version, CancellationToken cancellationToken);
        
        /// <summary>
        /// Update a new extension. It is allowed to update an extension which
        /// had already been registered with the same identity (i.e.
        /// combination of ProviderNamespace and Type) but with different
        /// version. It will fail if the extension to update has an identity
        /// that has not been registered before, or there is already an
        /// extension with the same identity and same version.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Virtual Machine Extension Image
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> BeginUpdatingAsync(ExtensionImageUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Register a new extension. An extension is identified by the
        /// combination of its ProviderNamespace and Type (case-sensitive
        /// string). It is not allowed to register an extension with the same
        /// identity (i.e. combination of ProviderNamespace and Type) of an
        /// already-registered extension. To register new version of an
        /// existing extension, the Update Extension API should be used.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Register Virtual Machine Extension Image
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        Task<OperationStatusResponse> RegisterAsync(ExtensionImageRegisterParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Unregister a version of an extension that was previously registered
        /// using either the Register Extension or Update Extension APIs. An
        /// extension version is identified by the combination of its
        /// ProviderNamespace, Type and Version which are specified when
        /// registering the extension. Unregistering is only allowed for
        /// internal extensions, that is, the extensions for which the
        /// IsInternalExtension field is set to 'true' during registration or
        /// during an update. There is a quota (15) on the number of
        /// extensions that can be registered per subscription. If your
        /// subscription runs out of quota, you will wither need to unregister
        /// some of the internal extensions or contact Azure (same email used
        /// to become a publisher)  to increase the quota.
        /// </summary>
        /// <param name='providerNamespace'>
        /// The provider namespace of the extension image to unregister.
        /// </param>
        /// <param name='type'>
        /// The type of the extension image to unregister.
        /// </param>
        /// <param name='version'>
        /// The version of the extension image to unregister.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        Task<OperationStatusResponse> UnregisterAsync(string providerNamespace, string type, string version, CancellationToken cancellationToken);
        
        /// <summary>
        /// Update a new extension. It is allowed to update an extension which
        /// had already been registered with the same identity (i.e.
        /// combination of ProviderNamespace and Type) but with different
        /// version. It will fail if the extension to update has an identity
        /// that has not been registered before, or there is already an
        /// extension with the same identity and same version.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Virtual Machine Extension Image
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        Task<OperationStatusResponse> UpdateAsync(ExtensionImageUpdateParameters parameters, CancellationToken cancellationToken);
    }
}
