// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Management.ExpressRoute;
using Microsoft.WindowsAzure.Management.ExpressRoute.Models;

namespace Microsoft.WindowsAzure.Management.ExpressRoute
{
    /// <summary>
    /// The Express Route API provides programmatic access to the functionality
    /// needed by the customer to set up Dedicated Circuits and Dedicated
    /// Circuit Links. The Express Route Customer API is a REST API. All API
    /// operations are performed over SSL and mutually authenticated using
    /// X.509 v3 certificates.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/ee460799.aspx for
    /// more information)
    /// </summary>
    public static partial class DedicatedCircuitLinkAuthorizationOperationsExtensions
    {
        /// <summary>
        /// The Remove Dedicated Circuit Link Authorization operation deletes
        /// an existing dedicated circuit link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. Service key representing the dedicated circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static ExpressRouteOperationResponse BeginRemove(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).BeginRemoveAsync(serviceKey, authId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Remove Dedicated Circuit Link Authorization operation deletes
        /// an existing dedicated circuit link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. Service key representing the dedicated circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<ExpressRouteOperationResponse> BeginRemoveAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return operations.BeginRemoveAsync(serviceKey, authId, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the link authorization for the specified dedicated circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// Get link authorization operation response.
        /// </returns>
        public static DedicatedCircuitLinkAuthorizationGetResponse Get(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).GetAsync(serviceKey, authId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the link authorization for the specified dedicated circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// Get link authorization operation response.
        /// </returns>
        public static Task<DedicatedCircuitLinkAuthorizationGetResponse> GetAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return operations.GetAsync(serviceKey, authId, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get Express Route operation status gets information on the
        /// status of Express Route operations in Windows Azure.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj154112.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='operationId'>
        /// Required. The id  of the operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static ExpressRouteOperationStatusResponse GetOperationStatus(this IDedicatedCircuitLinkAuthorizationOperations operations, string operationId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).GetOperationStatusAsync(operationId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get Express Route operation status gets information on the
        /// status of Express Route operations in Windows Azure.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj154112.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='operationId'>
        /// Required. The id  of the operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<ExpressRouteOperationStatusResponse> GetOperationStatusAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string operationId)
        {
            return operations.GetOperationStatusAsync(operationId, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists the specified link authorization for the specified dedicated
        /// circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <returns>
        /// List link authorization operation response.
        /// </returns>
        public static DedicatedCircuitLinkAuthorizationListResponse List(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).ListAsync(serviceKey);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists the specified link authorization for the specified dedicated
        /// circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <returns>
        /// List link authorization operation response.
        /// </returns>
        public static Task<DedicatedCircuitLinkAuthorizationListResponse> ListAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey)
        {
            return operations.ListAsync(serviceKey, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the specified link authorization for the specified dedicated
        /// circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the new Dedicated Circuit link
        /// authorization operation.
        /// </param>
        /// <returns>
        /// Get link authorization operation response.
        /// </returns>
        public static DedicatedCircuitLinkAuthorizationNewResponse New(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, DedicatedCircuitLinkAuthorizationNewParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).NewAsync(serviceKey, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the specified link authorization for the specified dedicated
        /// circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the new Dedicated Circuit link
        /// authorization operation.
        /// </param>
        /// <returns>
        /// Get link authorization operation response.
        /// </returns>
        public static Task<DedicatedCircuitLinkAuthorizationNewResponse> NewAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, DedicatedCircuitLinkAuthorizationNewParameters parameters)
        {
            return operations.NewAsync(serviceKey, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Remove Dedicated Circuit Link Authorization operation deletes
        /// an existing dedicated circuit link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. Service Key associated with the dedicated circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static ExpressRouteOperationStatusResponse Remove(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).RemoveAsync(serviceKey, authId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Remove Dedicated Circuit Link Authorization operation deletes
        /// an existing dedicated circuit link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. Service Key associated with the dedicated circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<ExpressRouteOperationStatusResponse> RemoveAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId)
        {
            return operations.RemoveAsync(serviceKey, authId, CancellationToken.None);
        }
        
        /// <summary>
        /// Updates the specified link authorization for the specified
        /// dedicated circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Dedicated Circuit link
        /// authorization operation.
        /// </param>
        /// <returns>
        /// Update link authorization operation response.
        /// </returns>
        public static DedicatedCircuitLinkAuthorizationUpdateResponse Update(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId, DedicatedCircuitLinkAuthorizationUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDedicatedCircuitLinkAuthorizationOperations)s).UpdateAsync(serviceKey, authId, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Updates the specified link authorization for the specified
        /// dedicated circuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.ExpressRoute.IDedicatedCircuitLinkAuthorizationOperations.
        /// </param>
        /// <param name='serviceKey'>
        /// Required. The service key representing the circuit.
        /// </param>
        /// <param name='authId'>
        /// Required. The GUID representing the authorization
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Dedicated Circuit link
        /// authorization operation.
        /// </param>
        /// <returns>
        /// Update link authorization operation response.
        /// </returns>
        public static Task<DedicatedCircuitLinkAuthorizationUpdateResponse> UpdateAsync(this IDedicatedCircuitLinkAuthorizationOperations operations, string serviceKey, string authId, DedicatedCircuitLinkAuthorizationUpdateParameters parameters)
        {
            return operations.UpdateAsync(serviceKey, authId, parameters, CancellationToken.None);
        }
    }
}
