// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using Microsoft.WindowsAzure.Management.WebSites.Models;

namespace Microsoft.WindowsAzure.Management.WebSites.Models
{
    /// <summary>
    /// Parameters supplied to the Create Web Site operation.
    /// </summary>
    public partial class WebSiteCreateParameters
    {
        private string _name;
        
        /// <summary>
        /// Required. The name of the web site. This should match the host name
        /// in the default domain.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private string _serverFarm;
        
        /// <summary>
        /// Required. The name of the Server Farm (Web Hosting Plan) associated
        /// with this web site. This is a required value. Server Farm must
        /// already exist.
        /// </summary>
        public string ServerFarm
        {
            get { return this._serverFarm; }
            set { this._serverFarm = value; }
        }
        
        private WebSiteCreateParameters.WebSpaceDetails _webSpace;
        
        /// <summary>
        /// Optional. Information about the web space to create.
        /// </summary>
        public WebSiteCreateParameters.WebSpaceDetails WebSpace
        {
            get { return this._webSpace; }
            set { this._webSpace = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteCreateParameters class.
        /// </summary>
        public WebSiteCreateParameters()
        {
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteCreateParameters class
        /// with required arguments.
        /// </summary>
        public WebSiteCreateParameters(string name, string serverFarm)
            : this()
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (serverFarm == null)
            {
                throw new ArgumentNullException("serverFarm");
            }
            this.Name = name;
            this.ServerFarm = serverFarm;
        }
        
        /// <summary>
        /// Information about the web space to create.
        /// </summary>
        public partial class WebSpaceDetails
        {
            private string _geoRegion;
            
            /// <summary>
            /// Required. The geographical region of the web space that will be
            /// created.
            /// </summary>
            public string GeoRegion
            {
                get { return this._geoRegion; }
                set { this._geoRegion = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// Required. The name of the web space.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private string _plan;
            
            /// <summary>
            /// Required. The web space plan. This value must be
            /// VirtualDedicatedPlan.
            /// </summary>
            public string Plan
            {
                get { return this._plan; }
                set { this._plan = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the WebSpaceDetails class.
            /// </summary>
            public WebSpaceDetails()
            {
            }
            
            /// <summary>
            /// Initializes a new instance of the WebSpaceDetails class with
            /// required arguments.
            /// </summary>
            public WebSpaceDetails(string geoRegion, string name, string plan)
                : this()
            {
                if (geoRegion == null)
                {
                    throw new ArgumentNullException("geoRegion");
                }
                if (name == null)
                {
                    throw new ArgumentNullException("name");
                }
                if (plan == null)
                {
                    throw new ArgumentNullException("plan");
                }
                this.GeoRegion = geoRegion;
                this.Name = name;
                this.Plan = plan;
            }
        }
    }
}
